@@ -1,4 +1,8 @@
-class OidcEndpointError(Exception):
+class OidcOPError(Exception):
+    pass
+
+
+class OidcEndpointError(OidcOPError):
     pass
 
 
@@ -96,7 +100,3 @@ class CapabilitiesMisMatch(OidcEndpointError):
 
 class MultipleCodeUsage(OidcEndpointError):
     pass
-
-
-class InvalidToken(Exception):
-    pass
